package org.dromara.system.controller.leave;

import java.util.List;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.system.domain.vo.AskLeaveVo;
import org.dromara.system.domain.bo.AskLeaveBo;
import org.dromara.system.service.IAskLeaveService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 请假
 *
 * @author boman
 * @date 2024-11-04
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/ask/leave")
public class AskLeaveController extends BaseController {

    private final IAskLeaveService askLeaveService;

    /**
     * 查询请假列表
     */
    @SaCheckPermission("ask:leave:list")
    @GetMapping("/list")
    public TableDataInfo<AskLeaveVo> list(AskLeaveBo bo, PageQuery pageQuery) {
        return askLeaveService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出请假列表
     */
    @SaCheckPermission("ask:leave:export")
    @Log(title = "请假", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(AskLeaveBo bo, HttpServletResponse response) {
        List<AskLeaveVo> list = askLeaveService.queryList(bo);
        ExcelUtil.exportExcel(list, "请假", AskLeaveVo.class, response);
    }

    /**
     * 获取请假详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("ask:leave:query")
    @GetMapping("/{id}")
    public R<AskLeaveVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(askLeaveService.queryById(id));
    }

    /**
     * 新增请假
     */
    @SaCheckPermission("ask:leave:add")
    @Log(title = "请假", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody AskLeaveBo bo) {
        return toAjax(askLeaveService.insertByBo(bo));
    }

    /**
     * 修改请假
     */
    @SaCheckPermission("ask:leave:edit")
    @Log(title = "请假", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PostMapping("/put")
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody AskLeaveBo bo) {
        return toAjax(askLeaveService.updateByBo(bo));
    }

    /**
     * 删除请假
     *
     * @param ids 主键串
     */
    /*@SaCheckPermission("ask:leave:remove")
    @Log(title = "请假", businessType = BusinessType.DELETE)
    @GetMapping("/delete/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(askLeaveService.deleteWithValidByIds(List.of(ids), true));
    }*/

    /**
     * 删除请假
     *
     */
    @SaCheckPermission("ask:leave:remove")
    @Log(title = "请假", businessType = BusinessType.DELETE)
    @GetMapping("/delete/{id}")
    public R<Void> remove(@NotNull(message = "主键不能为空")
                          @PathVariable Long id) {
        return toAjax(askLeaveService.deleteWithValidById(id, true));
    }

    /**
     * 审核
     */
    @SaCheckPermission("ask:leave:sh")
    @Log(title = "请假", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PostMapping("/sh")
    public R<Void> sh(@Validated(EditGroup.class) @RequestBody AskLeaveBo bo) {
        return toAjax(askLeaveService.sh(bo));
    }
}
