package org.dromara.system.controller.leave;

import java.util.List;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.system.domain.vo.AskLeaveFjVo;
import org.dromara.system.domain.bo.AskLeaveFjBo;
import org.dromara.system.service.IAskLeaveFjService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 请假附件
 *
 * @author boman
 * @date 2024-11-04
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/leaveFj")
public class AskLeaveFjController extends BaseController {

    private final IAskLeaveFjService askLeaveFjService;

    /**
     * 查询请假附件列表
     */
    @SaCheckPermission("system:leaveFj:list")
    @GetMapping("/list")
    public TableDataInfo<AskLeaveFjVo> list(AskLeaveFjBo bo, PageQuery pageQuery) {
        return askLeaveFjService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出请假附件列表
     */
    @SaCheckPermission("system:leaveFj:export")
    @Log(title = "请假附件", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(AskLeaveFjBo bo, HttpServletResponse response) {
        List<AskLeaveFjVo> list = askLeaveFjService.queryList(bo);
        ExcelUtil.exportExcel(list, "请假附件", AskLeaveFjVo.class, response);
    }

    /**
     * 获取请假附件详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("system:leaveFj:query")
    @GetMapping("/{id}")
    public R<AskLeaveFjVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(askLeaveFjService.queryById(id));
    }

    /**
     * 新增请假附件
     */
    @SaCheckPermission("system:leaveFj:add")
    @Log(title = "请假附件", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody AskLeaveFjBo bo) {
        return toAjax(askLeaveFjService.insertByBo(bo));
    }

    /**
     * 修改请假附件
     */
    @SaCheckPermission("system:leaveFj:edit")
    @Log(title = "请假附件", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PostMapping("/put")
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody AskLeaveFjBo bo) {
        return toAjax(askLeaveFjService.updateByBo(bo));
    }

    /**
     * 删除请假附件
     *
     * @param ids 主键串
     */
    @SaCheckPermission("system:leaveFj:remove")
    @Log(title = "请假附件", businessType = BusinessType.DELETE)
    @GetMapping("/delete/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(askLeaveFjService.deleteWithValidByIds(List.of(ids), true));
    }
}
