package org.dromara.system.controller.register;

import java.util.List;

import cn.dev33.satoken.annotation.SaIgnore;
import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.system.domain.vo.RegisterParentsVo;
import org.dromara.system.domain.bo.RegisterParentsBo;
import org.dromara.system.service.IRegisterParentsService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 注册-家长
 *
 * @author Lion Li
 * @date 2023-08-16
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/register/parents")
public class RegisterParentsController extends BaseController {

    private final IRegisterParentsService registerParentsService;

    /**
     * 查询注册-家长列表
     */
    @SaCheckPermission("register:parents:list")
    @GetMapping("/list")
    public TableDataInfo<RegisterParentsVo> list(RegisterParentsBo bo, PageQuery pageQuery) {
        return registerParentsService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出注册-家长列表
     */
    @SaCheckPermission("register:parents:export")
    @Log(title = "注册-家长", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(RegisterParentsBo bo, HttpServletResponse response) {
        List<RegisterParentsVo> list = registerParentsService.queryList(bo);
        ExcelUtil.exportExcel(list, "注册-家长", RegisterParentsVo.class, response);
    }

    /**
     * 获取注册-家长详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("register:parents:query")
    @GetMapping("/{id}")
    public R<RegisterParentsVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(registerParentsService.queryById(id));
    }

    /**
     * 新增注册-家长
     */
    @Log(title = "注册-家长", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<RegisterParentsVo> add(@Validated(AddGroup.class) @RequestBody RegisterParentsBo bo) {
        return registerParentsService.insertByBo(bo);
    }

    /**
     * 修改注册-家长
     */
    @SaCheckPermission("register:parents:edit")
    @Log(title = "注册-家长", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PostMapping("/put")
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody RegisterParentsBo bo) {
        return toAjax(registerParentsService.updateByBo(bo));
    }

    /**
     * 删除注册-家长
     *
     * @param ids 主键串
     */
    @SaCheckPermission("register:parents:remove")
    @Log(title = "注册-家长", businessType = BusinessType.DELETE)
    @GetMapping("/delete/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(registerParentsService.deleteWithValidByIds(List.of(ids), true));
    }
}
