package org.dromara.system.controller.register;

import java.util.List;
import java.util.Map;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.system.domain.vo.RegisterSchoolVo;
import org.dromara.system.domain.bo.RegisterSchoolBo;
import org.dromara.system.service.IRegisterSchoolService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 注册-学校
 *
 * @author boman
 * @date 2023-08-16
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/register/school")
public class RegisterSchoolController extends BaseController {

    private final IRegisterSchoolService registerSchoolService;

    /**
     * 查询注册-学校列表
     */
    @SaCheckPermission("register:school:list")
    @GetMapping("/list")
    public TableDataInfo<RegisterSchoolVo> list(RegisterSchoolBo bo, PageQuery pageQuery) {
        return registerSchoolService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出注册-学校列表
     */
    @SaCheckPermission("register:school:export")
    @Log(title = "注册-学校", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(RegisterSchoolBo bo, HttpServletResponse response) {
        List<RegisterSchoolVo> list = registerSchoolService.queryList(bo);
        ExcelUtil.exportExcel(list, "注册-学校", RegisterSchoolVo.class, response);
    }

    /**
     * 获取注册-学校详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("register:school:query")
    @GetMapping("/{id}")
    public R<RegisterSchoolVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(registerSchoolService.queryById(id));
    }

    /**
     * 新增注册-学校
     */
    @Log(title = "注册-学校", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<RegisterSchoolVo> add(@Validated(AddGroup.class) @RequestBody RegisterSchoolBo bo) {
        return registerSchoolService.insertByBo(bo);
    }

    /**
     * 修改注册-学校
     */
    @SaCheckPermission("register:school:edit")
    @Log(title = "注册-学校", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PostMapping("/put")
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody RegisterSchoolBo bo) {
        return toAjax(registerSchoolService.updateByBo(bo));
    }

    /**
     * 删除注册-学校
     *
     * @param ids 主键串
     */
    @SaCheckPermission("register:school:remove")
    @Log(title = "注册-学校", businessType = BusinessType.DELETE)
    @GetMapping("/delete/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(registerSchoolService.deleteWithValidByIds(List.of(ids), true));
    }

    /**
     * 注册信息查询
     */
    @GetMapping(value = "/query")
    public R<Map<String,Object>> queryInfo(String phonenumber)
    {
        return registerSchoolService.queryInfo(phonenumber);
    }
}
