package org.dromara.system.controller.register;

import java.util.List;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.system.domain.vo.RegisterSchoolFileVo;
import org.dromara.system.domain.bo.RegisterSchoolFileBo;
import org.dromara.system.service.IRegisterSchoolFileService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 注册-学校-附件
 *
 * @author boman
 * @date 2023-08-16
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/register/file")
public class RegisterSchoolFileController extends BaseController {

    private final IRegisterSchoolFileService registerSchoolFileService;

    /**
     * 查询注册-学校-附件列表
     */
    @SaCheckPermission("register:file:list")
    @GetMapping("/list")
    public TableDataInfo<RegisterSchoolFileVo> list(RegisterSchoolFileBo bo, PageQuery pageQuery) {
        return registerSchoolFileService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出注册-学校-附件列表
     */
    @SaCheckPermission("register:file:export")
    @Log(title = "注册-学校-附件", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(RegisterSchoolFileBo bo, HttpServletResponse response) {
        List<RegisterSchoolFileVo> list = registerSchoolFileService.queryList(bo);
        ExcelUtil.exportExcel(list, "注册-学校-附件", RegisterSchoolFileVo.class, response);
    }

    /**
     * 获取注册-学校-附件详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("register:file:query")
    @GetMapping("/{id}")
    public R<RegisterSchoolFileVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(registerSchoolFileService.queryById(id));
    }

    /**
     * 新增注册-学校-附件
     */
    @SaCheckPermission("register:file:add")
    @Log(title = "注册-学校-附件", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody RegisterSchoolFileBo bo) {
        return toAjax(registerSchoolFileService.insertByBo(bo));
    }

    /**
     * 修改注册-学校-附件
     */
    @SaCheckPermission("register:file:edit")
    @Log(title = "注册-学校-附件", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PostMapping("/put")
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody RegisterSchoolFileBo bo) {
        return toAjax(registerSchoolFileService.updateByBo(bo));
    }

    /**
     * 删除注册-学校-附件
     *
     * @param ids 主键串
     */
    @SaCheckPermission("register:file:remove")
    @Log(title = "注册-学校-附件", businessType = BusinessType.DELETE)
    @GetMapping("/delete/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(registerSchoolFileService.deleteWithValidByIds(List.of(ids), true));
    }
}
