package org.dromara.system.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.dromara.common.core.constant.CacheNames;
import org.dromara.common.core.constant.UserConstants;
import org.dromara.common.core.domain.dto.RoleDTO;
import org.dromara.common.core.domain.model.LoginUser;
import org.dromara.common.core.exception.ServiceException;
import org.dromara.common.core.service.DeptService;
import org.dromara.common.core.utils.*;
import org.dromara.common.mybatis.helper.DataBaseHelper;
import org.dromara.common.redis.utils.CacheUtils;
import org.dromara.common.redis.utils.RedisUtils;
import org.dromara.common.satoken.utils.LoginHelper;
import org.dromara.system.domain.SysDept;
import org.dromara.system.domain.SysRole;
import org.dromara.system.domain.SysUser;
import org.dromara.system.domain.bo.FormalTeacherClassBo;
import org.dromara.system.domain.bo.SysDeptBo;
import org.dromara.system.domain.vo.FormalTeacherClassVo;
import org.dromara.system.domain.vo.SysDeptVo;
import org.dromara.system.domain.xiake.XiakeConfig;
import org.dromara.system.mapper.FormalTeacherClassMapper;
import org.dromara.system.mapper.SysDeptMapper;
import org.dromara.system.mapper.SysRoleMapper;
import org.dromara.system.mapper.SysUserMapper;
import org.dromara.system.mapper.xiake.XiakeConfigMapper;
import org.dromara.system.service.IFormalTeacherClassService;
import org.dromara.system.service.ISysDeptService;
import lombok.RequiredArgsConstructor;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 部门管理 服务实现
 *
 * @author Lion Li
 */
@RequiredArgsConstructor
@Service
public class SysDeptServiceImpl implements ISysDeptService, DeptService {

    private final SysDeptMapper baseMapper;
    private final SysRoleMapper roleMapper;
    private final SysUserMapper userMapper;

    private final IFormalTeacherClassService formalTeacherClassService;
    private final FormalTeacherClassMapper formalTeacherClassMapper;

    private final XiakeConfigMapper configMapper;

    /**
     * 查询部门管理数据
     *
     * @param dept 部门信息
     * @return 部门信息集合
     */
    @Override
    public List<SysDeptVo> selectDeptList(SysDeptBo dept) {
        LambdaQueryWrapper<SysDept> lqw = buildQueryWrapper(dept);
        return baseMapper.selectDeptList(lqw);
    }


    /**
     * 查询部门树结构信息
     *
     * @param bo 部门信息
     * @return 部门树信息集合
     */
    @Override
    public List<Tree<Long>> selectDeptTreeList(SysDeptBo bo) {
        // 只查询未禁用部门
        bo.setStatus(UserConstants.DEPT_NORMAL);
        LambdaQueryWrapper<SysDept> lqw = buildQueryWrapper(bo);
        List<SysDeptVo> depts = baseMapper.selectDeptList(lqw);
        return buildDeptTreeSelect(depts);
    }

    private LambdaQueryWrapper<SysDept> buildQueryWrapper(SysDeptBo bo) {
        LambdaQueryWrapper<SysDept> lqw = Wrappers.lambdaQuery();
        lqw.eq(SysDept::getDelFlag, "0");
        lqw.eq(ObjectUtil.isNotNull(bo.getDeptId()), SysDept::getDeptId, bo.getDeptId());
        lqw.eq(ObjectUtil.isNotNull(bo.getParentId()), SysDept::getParentId, bo.getParentId());
        lqw.like(StringUtils.isNotBlank(bo.getDeptName()), SysDept::getDeptName, bo.getDeptName());
        lqw.eq(StringUtils.isNotBlank(bo.getStatus()), SysDept::getStatus, bo.getStatus());
        lqw.orderByAsc(SysDept::getParentId);
        lqw.orderByAsc(SysDept::getOrderNum);
        return lqw;
    }

    /**
     * 构建前端所需要下拉树结构
     *
     * @param depts 部门列表
     * @return 下拉树结构列表
     */
    @Override
    public List<Tree<Long>> buildDeptTreeSelect(List<SysDeptVo> depts) {
        if (CollUtil.isEmpty(depts)) {
            return CollUtil.newArrayList();
        }
        return TreeBuildUtils.build(depts, (dept, tree) ->
            tree.setId(dept.getDeptId())
                .setParentId(dept.getParentId())
                .setName(dept.getDeptName())
                .setWeight(dept.getOrderNum()));
    }

    /**
     * 根据角色ID查询部门树信息
     *
     * @param roleId 角色ID
     * @return 选中部门列表
     */
    @Override
    public List<Long> selectDeptListByRoleId(Long roleId) {
        SysRole role = roleMapper.selectById(roleId);
        return baseMapper.selectDeptListByRoleId(roleId, role.getDeptCheckStrictly());
    }

    /**
     * 根据部门ID查询信息
     *
     * @param deptId 部门ID
     * @return 部门信息
     */
    @Cacheable(cacheNames = CacheNames.SYS_DEPT, key = "#deptId")
    @Override
    public SysDeptVo selectDeptById(Long deptId) {
        SysDeptVo dept = baseMapper.selectVoById(deptId);
        if (ObjectUtil.isNull(dept)) {
            return null;
        }
        SysDeptVo parentDept = baseMapper.selectVoOne(new LambdaQueryWrapper<SysDept>()
            .select(SysDept::getDeptName).eq(SysDept::getDeptId, dept.getParentId()));
        dept.setParentName(ObjectUtil.isNotNull(parentDept) ? parentDept.getDeptName() : null);
        return dept;
    }

    /**
     * 通过部门ID查询部门名称
     *
     * @param deptIds 部门ID串逗号分隔
     * @return 部门名称串逗号分隔
     */
    @Override
    public String selectDeptNameByIds(String deptIds) {
        List<String> list = new ArrayList<>();
        for (Long id : StringUtils.splitTo(deptIds, Convert::toLong)) {
            SysDeptVo vo = SpringUtils.getAopProxy(this).selectDeptById(id);
            if (ObjectUtil.isNotNull(vo)) {
                list.add(vo.getDeptName());
            }
        }
        return String.join(StringUtils.SEPARATOR, list);
    }

    /**
     * 根据ID查询所有子部门数（正常状态）
     *
     * @param deptId 部门ID
     * @return 子部门数
     */
    @Override
    public long selectNormalChildrenDeptById(Long deptId) {
        return baseMapper.selectCount(new LambdaQueryWrapper<SysDept>()
            .eq(SysDept::getStatus, UserConstants.DEPT_NORMAL)
            .apply(DataBaseHelper.findInSet(deptId, "ancestors")));
    }

    /**
     * 是否存在子节点
     *
     * @param deptId 部门ID
     * @return 结果
     */
    @Override
    public boolean hasChildByDeptId(Long deptId) {
        return baseMapper.exists(new LambdaQueryWrapper<SysDept>()
            .eq(SysDept::getParentId, deptId));
    }

    /**
     * 查询部门是否存在用户
     *
     * @param deptId 部门ID
     * @return 结果 true 存在 false 不存在
     */
    @Override
    public boolean checkDeptExistUser(Long deptId) {
        return userMapper.exists(new LambdaQueryWrapper<SysUser>()
            .eq(SysUser::getDeptId, deptId));
    }

    /**
     * 校验部门名称是否唯一
     *
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public boolean checkDeptNameUnique(SysDeptBo dept) {
        boolean exist = baseMapper.exists(new LambdaQueryWrapper<SysDept>()
            .eq(SysDept::getDeptName, dept.getDeptName())
            .eq(SysDept::getParentId, dept.getParentId())
            .ne(ObjectUtil.isNotNull(dept.getDeptId()), SysDept::getDeptId, dept.getDeptId()));
        return !exist;
    }

    /**
     * 校验部门是否有数据权限
     *
     * @param deptId 部门id
     */
    @Override
    public void checkDeptDataScope(Long deptId) {
        if (ObjectUtil.isNull(deptId)) {
            return;
        }
        if (LoginHelper.isSuperAdmin()) {
            return;
        }
        SysDeptVo dept = baseMapper.selectDeptById(deptId);
        if (ObjectUtil.isNull(dept)) {
            throw new ServiceException("没有权限访问部门数据！");
        }
    }

    /**
     * 新增保存部门信息
     *
     * @param bo 部门信息
     * @return 结果
     */
    @Override
    @Transactional
    public int insertDept(SysDeptBo bo) {
        SysDept info = baseMapper.selectById(bo.getParentId());
        // 如果父节点不为正常状态,则不允许新增子节点
        if (!UserConstants.DEPT_NORMAL.equals(info.getStatus())) {
            throw new ServiceException("部门停用，不允许新增");
        }
        SysDept dept = MapstructUtils.convert(bo, SysDept.class);
        dept.setAncestors(info.getAncestors() + StringUtils.SEPARATOR + dept.getParentId());
        int i = baseMapper.insert(dept);
        if(info!=null){
            //往xiake_config插入数据
            XiakeConfig config = new XiakeConfig();
            config.setConfigName("下课时间");
            config.setConfigKey(dept.getDeptId()+":1");
            config.setConfigValue("5");
            config.setConfigType("1");
            config.setCreateTime(DateUtils.getNowDate());
            config.setUpdateTime(DateUtils.getNowDate());
            configMapper.insert(config);
            XiakeConfig config1 = new XiakeConfig();
            config1.setConfigName("延迟下课");
            config1.setConfigKey(dept.getDeptId()+":2");
            config1.setConfigValue("5");
            config.setConfigType("2");
            config1.setCreateTime(DateUtils.getNowDate());
            config1.setUpdateTime(DateUtils.getNowDate());
            configMapper.insert(config1);

            String key = dept.getParentId() + ":" + dept.getDeptId();
            String value = dept.getDeptName() + ":time";
            RedisUtils.setCacheObject(key, value);
        }

        return i;
    }

    /**
     * 修改保存部门信息
     *
     * @param bo 部门信息
     * @return 结果
     */
    @CacheEvict(cacheNames = CacheNames.SYS_DEPT, key = "#bo.deptId")
    @Override
    public int updateDept(SysDeptBo bo) {
        SysDept dept = MapstructUtils.convert(bo, SysDept.class);
        SysDept oldDept = baseMapper.selectById(dept.getDeptId());
        if (!oldDept.getParentId().equals(dept.getParentId())) {
            // 如果是新父部门 则校验是否具有新父部门权限 避免越权
            this.checkDeptDataScope(dept.getParentId());
            SysDept newParentDept = baseMapper.selectById(dept.getParentId());
            if (ObjectUtil.isNotNull(newParentDept) && ObjectUtil.isNotNull(oldDept)) {
                String newAncestors = newParentDept.getAncestors() + StringUtils.SEPARATOR + newParentDept.getDeptId();
                String oldAncestors = oldDept.getAncestors();
                dept.setAncestors(newAncestors);
                updateDeptChildren(dept.getDeptId(), newAncestors, oldAncestors);
            }
        }
        int result = baseMapper.updateById(dept);
        if (UserConstants.DEPT_NORMAL.equals(dept.getStatus()) && StringUtils.isNotEmpty(dept.getAncestors())
            && !StringUtils.equals(UserConstants.DEPT_NORMAL, dept.getAncestors())) {
            // 如果该部门是启用状态，则启用该部门的所有上级部门
            updateParentDeptStatusNormal(dept);
        }
        return result;
    }

    /**
     * 修改该部门的父级部门状态
     *
     * @param dept 当前部门
     */
    private void updateParentDeptStatusNormal(SysDept dept) {
        String ancestors = dept.getAncestors();
        Long[] deptIds = Convert.toLongArray(ancestors);
        baseMapper.update(null, new LambdaUpdateWrapper<SysDept>()
            .set(SysDept::getStatus, UserConstants.DEPT_NORMAL)
            .in(SysDept::getDeptId, Arrays.asList(deptIds)));
    }

    /**
     * 修改子元素关系
     *
     * @param deptId       被修改的部门ID
     * @param newAncestors 新的父ID集合
     * @param oldAncestors 旧的父ID集合
     */
    private void updateDeptChildren(Long deptId, String newAncestors, String oldAncestors) {
        List<SysDept> children = baseMapper.selectList(new LambdaQueryWrapper<SysDept>()
            .apply(DataBaseHelper.findInSet(deptId, "ancestors")));
        List<SysDept> list = new ArrayList<>();
        for (SysDept child : children) {
            SysDept dept = new SysDept();
            dept.setDeptId(child.getDeptId());
            dept.setAncestors(child.getAncestors().replaceFirst(oldAncestors, newAncestors));
            list.add(dept);
        }
        if (CollUtil.isNotEmpty(list)) {
            if (baseMapper.updateBatchById(list)) {
                list.forEach(dept -> CacheUtils.evict(CacheNames.SYS_DEPT, dept.getDeptId()));
            }
        }
    }

    /**
     * 删除部门管理信息
     *
     * @param deptId 部门ID
     * @return 结果
     */
    @CacheEvict(cacheNames = CacheNames.SYS_DEPT, key = "#deptId")
    @Override
    public int deleteDeptById(Long deptId) {
        return baseMapper.deleteById(deptId);
    }

    @Override
    public List<SysDeptVo> selectDeptOldList(SysDeptBo dept) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        if(!"admin".equals(loginUser.getUsername())){
            dept.setTenantId(loginUser.getTenantId());
            dept.setParentId(loginUser.getDeptId());
            if(loginUser.getDeptId()==null || loginUser.getDeptId() == 0L){
                List<FormalTeacherClassVo> teacherClass = formalTeacherClassService.getTeacherClass(String.valueOf(loginUser.getUserId()));
                dept.setParentId(teacherClass.get(0).getSchoolId());
            }
        }
        return baseMapper.selectDeptOldList(dept);
    }


    @Override
    public List<SysDeptVo> selectDeptListChild(SysDeptBo dept) {
        LoginUser user = LoginHelper.getLoginUser();
        if(!"admin".equals(user.getUsername())){
            if(dept.getParentId()==null ||dept.getParentId()==0L){
                dept.setParentId(user.getDeptId());
            }
            dept.setDeptId(user.getDeptId());
            if(user.getDeptId()==null || user.getDeptId() == 0L){
                List<FormalTeacherClassVo> teacherClass = formalTeacherClassService.getTeacherClass(String.valueOf(user.getUserId()));
                if(dept.getParentId()==null ||dept.getParentId()==0L){
                    dept.setParentId(teacherClass.get(0).getSchoolId());
                }
                dept.setDeptId(teacherClass.get(0).getSchoolId());
            }
        }
        return baseMapper.selectDeptListMapper(dept);
    }

    @Override
    public List<SysDeptVo> selectDeptListChildren(Long parentId) {
        return baseMapper.selectDeptListChildren(parentId);
    }


    @Override
    public List<SysDeptVo> pcSelfList(SysDeptBo dept) {
        LoginUser loginUser = LoginHelper.getLoginUser();
        List<RoleDTO> roles = loginUser.getRoles();

        if("admin".equals(loginUser.getUsername())){
            return baseMapper.selectDeptSelfListMapper(dept);
        }else {
            int type = 0;
            for (RoleDTO role : roles) {
                if("school".equals(role.getRoleKey())){
                    type = 1;
                    break;
                }else if("teacher".equals(role.getRoleKey())){
                    type = 2;
                }
            }
            if(type==1){
                if(dept.getParentId()==null || dept.getParentId()==0L){
                    dept.setDeptId(loginUser.getDeptId());
                }
                return baseMapper.selectDeptSelfListMapper(dept);
            }else if(type==2){
                //查询老师的班级
                FormalTeacherClassBo bo = new FormalTeacherClassBo();
                bo.setTeacherId(loginUser.getUserId());
                List<FormalTeacherClassVo> teacherClass = formalTeacherClassService.getTeacherClass(String.valueOf(loginUser.getUserId()));
                List<Long> deptIdList = teacherClass.stream().map(FormalTeacherClassVo::getClassId).collect(Collectors.toList());
                if(dept.getParentId()==null || dept.getParentId()==0L){
                    deptIdList.add(loginUser.getDeptId());
                }
                return baseMapper.selectDeptListMapperByIdList(deptIdList);
            }
            return new ArrayList<>();
        }
    }


    @Override
    public List<SysDeptVo> appSelfList(SysDeptBo dept) {
        LoginUser loginUser = LoginHelper.getLoginUser();

        Map<String, Object> params = dept.getParams();
        String role = "";
        if (params != null && params.size() > 0) {
            role = (String) params.get("role");
        }
        if("admin".equals(loginUser.getUsername())){
            return baseMapper.selectDeptOldList(dept);
        }else {
            if("school".equals(role)){
                dept.setDeptId(loginUser.getDeptId());
                dept.setParentId(loginUser.getDeptId());
                return baseMapper.selectDeptOldList(dept);
            }else if("teacher".equals(role)){
                //查询老师的班级
                FormalTeacherClassBo bo = new FormalTeacherClassBo();
                bo.setTeacherId(loginUser.getUserId());
                List<FormalTeacherClassVo> teacherClass = formalTeacherClassService.getTeacherClass(String.valueOf(loginUser.getUserId()));
                List<Long> deptIdList = teacherClass.stream().map(FormalTeacherClassVo::getClassId).collect(Collectors.toList());
                //deptIdList.add(loginUser.getDeptId());
                return baseMapper.selectDeptListMapperByIdList(deptIdList);
            }
            return new ArrayList<>();
        }
    }


}
